import glob
import os
import subprocess  # nosec
import sys
import tempfile
from typing import Dict, Optional

from setuptools import build_meta as _orig

prepare_metadata_for_build_wheel = _orig.prepare_metadata_for_build_wheel
build_sdist = _orig.build_sdist

VERSION = "3.20.1"


def get_requires_for_build_wheel(self, config_settings=None):  # type: ignore[no-untyped-def]
    return _orig.get_requires_for_build_wheel(config_settings)


def get_requires_for_build_sdist(self, config_settings=None):  # type: ignore[no-untyped-def]
    return _orig.get_requires_for_build_sdist(config_settings)


def build_wheel(  # type: ignore[no-untyped-def]
    wheel_directory, config_settings, metadata_directory=None
):
    python_bin = sys.executable
    cpu_count = os.cpu_count() or 4
    wheel_directory = os.path.abspath(wheel_directory)
    with tempfile.TemporaryDirectory(dir=wheel_directory) as tmp_dist_dir:
        run_command(
            f"git clone --depth 1 --branch v{VERSION} https://github.com/protocolbuffers/protobuf {tmp_dist_dir}/protobuf"
        )
        run_command(f"cd {tmp_dist_dir}/protobuf && ./autogen.sh")
        run_command(f"cd {tmp_dist_dir}/protobuf && ./configure")
        run_command(f"cd {tmp_dist_dir}/protobuf && make -j{cpu_count}")
        run_command(
            f"cd {tmp_dist_dir}/protobuf/python && MAKEFLAGS=-j{cpu_count} LD_LIBRARY_PATH=../src/.libs {python_bin} setup.py build --cpp_implementation"
        )
        run_command(
            f"cd {tmp_dist_dir}/protobuf/python && MAKEFLAGS=-j{cpu_count} LD_LIBRARY_PATH=../src/.libs {python_bin} setup.py bdist_wheel --cpp_implementation"
        )
        wheels = glob.glob(f"{tmp_dist_dir}/protobuf/python/dist/*.whl")[0]
        wheel_file = wheels[0]
        result_basename = os.path.basename(wheel_file)
        target_dir = wheel_directory
        result_path = os.path.join(target_dir, result_basename)
        os.rename(wheel_file, result_path)
    return result_basename


def run_command(
    cmd: str, env: Optional[Dict[str, str]] = None, timeout: Optional[int] = None
) -> None:
    """Implement subprocess.run but handle timeout different."""
    subprocess.run(
        cmd,
        shell=True,  # nosec
        check=True,
        stdout=sys.stdout,
        stderr=sys.stderr,
        env=env,
        timeout=timeout,
    )



#!/usr/bin/env python

"""The setup script."""

from setuptools import find_packages, setup

requirements = ["protobuf>=3.12.2,<4.0"]

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest>=3",
]

setup(
    author="J. Nick Koston",
    author_email="nick@koston.org",
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    description="Python library for building protobuf wheels",
    install_requires=requirements,
    license="Apache Software License 2.0",
    include_package_data=True,
    keywords="protobuf protobuf-wheel-builder",
    name="protobuf-wheel-builder",
    build_requires=[],
    packages=find_packages(include=["protobuf_wheel_builder"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/bdraco/protobuf-wheel-builder",
    version="1.0.0",
    zip_safe=False,
)
